package me.km.inventory;

import net.minecraft.entity.Entity;
import net.minecraft.inventory.InventoryBasic;
import net.minecraft.item.ItemStack;

public class InventoryBase extends InventoryBasic
{
    private final Entity owner;
    private final int rows;
    private final byte[] data;
    
    public InventoryBase(String title, byte[] data, int slots, Entity owner) 
    {
        super(title, true, slots);
        this.owner = owner;
        this.rows = slots / 9 + (slots % 9 == 0 ? 0 : 1);
        this.data = data;
    }
    
    private static int countZeros(String slots)
    {
        int i = 0;
        for(char c : slots.toCharArray())
        {
            if(c == '0')
            {
                i++;
            }
        }
        return i;
    }
    
    public InventoryBase(String title, String slots, Entity owner) 
    {
        super(title, true, Math.min(54, slots.length() - countZeros(slots)));
        this.owner = owner;
        
        int slotAmount = Math.min(54, slots.length());
        this.rows = slotAmount / 9 + (slotAmount % 9 == 0 ? 0 : 1);
        
        this.data = new byte[14];
        
        int id;
        for(int i = 0; i < slotAmount; i++)
        {
            id = Character.getNumericValue(slots.charAt(i));
            if(id > 3 || id < 0)
            {
                id = 0;
            }
            switch(i % 4)
            {
                case 0:
                    data[i / 4] += id;
                    break;
                case 1:
                    data[i / 4] += id << 2;
                    break;
                case 2:
                    data[i / 4] += id << 4;
                    break;
                case 3:
                    data[i / 4] += id << 6;
                    break;
            }
        }
    }
    
    public InventoryBase(String title, int slots, Entity owner) 
    {
        super(title, true, Math.max(9, Math.min(54, ((slots / 9) + ((slots % 9) == 0 ? 0 : 1)) * 9)));
        this.owner = owner;
        this.rows = super.getSizeInventory() / 9;
        this.data = new byte[14];
        
        int end = super.getSizeInventory();
        for(int i = 0; i < end; i++)
        {
            switch(i % 4)
            {
                case 0:
                    data[i / 4] += 1;
                    break;
                case 1:
                    data[i / 4] += 4;
                    break;
                case 2:
                    data[i / 4] += 16;
                    break;
                case 3:
                    data[i / 4] += 64;
                    break;
            }
        }
    }

    public byte[] getData() 
    {
        return data;
    }

    public Entity getOwner() 
    {
        return owner;
    }
    
    public void setContents(Iterable<? extends ItemStack> stacks)
    {
        int counter = 0;
        int size = super.getSizeInventory();
        for(ItemStack stack : stacks)
        {
            this.setInventorySlotContents(counter, stack);
            counter++;
            if(counter >= size)
            {
                return;
            }
        }
    }
    
    public int getRows()
    {
        return rows;
    }
    
    public int getSlotStatus(int slot)
    {
        int index = Math.max(0, Math.min(13, slot / 4));
        switch(slot % 4)
        {
            case 0: return data[index] & 3;
            case 1: return (data[index] >> 2) & 3;
            case 2: return (data[index] >> 4) & 3;
            case 3: return (data[index] >> 6) & 3;
        }
        return 0;
    }
    
    public int getSlotStatus(int x, int y)
    {
        return getSlotStatus(y * 9 + x);
    }
    
    public boolean isSlotValid(int slot)
    {
        return getSlotStatus(slot) != 0;
    }
    
    public boolean isSlotValid(int x, int y)
    {
        return getSlotStatus(x, y) != 0;
    }
}
